'From Smalltalk/X, Version:5.2.8 on 14-01-2008 at 02:12:09 PM'                  !

"{ Package: 'SpreadSheet:simekf1' }"

TestCase subclass:#ExpressionTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'obp-SpreadSheet'
!

!ExpressionTests methodsFor:'testing'!

test01
        " just expression tests with no cell references "
        | tree branch |

        " something ordinary "
        " 3 + (2 * 2) "
        branch := ExprBinTimes with: 2 with: 2.
        tree := ExprBinPlus with: 3 with: branch.
        self assert: (tree value = 7).

        " 1.25 * (4 / 2) - (-1) "
        branch := ExprBinOver with: 4 with: 2.
        branch := ExprBinTimes with: 1.25 with: branch.
        tree := ExprBinMinus with: branch with: (ExprUnNegation with: 1).
        self assert: (tree value = 3.5).

        " something weird: "
        " 'foobar' * 2 + Date(0) "
        branch := ExprBinTimes with: 'foobar' with: 2.
        tree := ExprBinPlus with: branch with: (Date fromDays: 0).
        self assert: (tree value = 'foobarfoobar1901-1-1').

        " something meaningless "
        tree := ExprBinTimes with: (Date fromDays: 30) with: (ExprBinPlus with: 1 with: 1.5).
        self assert: (tree value isNil).
!

test02
    " expression tests with cell references "
    
    |tree branch sheet|

    sheet := SpreadSheet new.
    sheet appendRow:#( 2 2 3 ).
    sheet appendRow:#( 4 2 1.25 1 ).
    sheet appendRow:#( 'foobar' ).
    sheet 
        cellAtRow:3
        column:2
        put:(Date fromDays:0).

     " something ordinary " " 3 + (2 * 2) "
    branch := ExprBinTimes with:(CellReference 
                        newAtRow:1
                        column:1
                        atSheet:sheet)
                with:(CellReference 
                        newAtRow:1
                        column:2
                        atSheet:sheet).
    tree := ExprBinPlus with:(CellReference 
                        newAtRow:1
                        column:3
                        atSheet:sheet)
                with:branch.
    self assert:(tree value = 7).

     " 1.25 * (4 / 2) - (-1) "
    branch := ExprBinOver with:(CellReference 
                        newAtRow:2
                        column:1
                        atSheet:sheet)
                with:(CellReference 
                        newAtRow:2
                        column:2
                        atSheet:sheet).
    branch := ExprBinTimes with:(CellReference 
                        newAtRow:2
                        column:3
                        atSheet:sheet)
                with:branch.
    tree := ExprBinMinus with:branch
                with:(ExprUnNegation with:(CellReference 
                                newAtRow:2
                                column:4
                                atSheet:sheet)).
    self assert:(tree value = 3.5).

     " something weird: " " 'foobar' * 2 + Date(0) "
    branch := ExprBinTimes with:(CellReference 
                        newAtRow:3
                        column:1
                        atSheet:sheet)
                with:(CellReference 
                        newAtRow:1
                        column:1
                        atSheet:sheet).
    tree := ExprBinPlus with:branch with:(Date fromDays:0).
    self assert:(tree value = 'foobarfoobar1901-1-1').

     " something meaningless "
    tree := ExprBinTimes with:(CellReference 
                        newAtRow:3
                        column:2
                        atSheet:sheet)
                with:(ExprBinPlus with:1 with:1.5).
    self assert:(tree value isNil).
!

test03
        " tests expressions with functions "
        | ref fun sheet |

        sheet := SpreadSheet new.
        sheet cellAtRow: 1 column: 1 put: 5.
        sheet cellAtRow: 1 column: 2 put: 7.
        sheet cellAtRow: 1 column: 3 put: -1.
        sheet cellAtRow: 1 column: 4 put: -2.
        sheet cellAtRow: 2 column: 1 put: -7.
        sheet cellAtRow: 3 column: 1 put: 2.

        fun := ExprFunSum new.
        ref := CellReferenceRange new.
        ref setReferenceFromR: 1 C: 1 toR: 1 C: 4 atSheet: sheet.
        fun operands: ref.
        self assert: (fun value = 9).

        ref := CellReferenceRange new.
        ref setReferenceFromR: 1 C: 1 toR: 3 C: 1 atSheet: sheet.
        fun operands: ref.
        self assert: (fun value = 0).

        " how does a sum react to a nil value? "
        ref := CellReferenceRange new.
        ref setReferenceFromR: 3 C: 1 toR: 4 C: 2 atSheet: sheet.
        fun operands: ref.
        self assert: (fun value = 2).
!

test04
        " tests expressions built up from the basic CellExpression class
          i.e. - tests the link between CellExpression and an expression built "
        | sheet cell expr |

        sheet := SpreadSheet new.
        cell := CellExpression new.
        expr := ExprBinPlus with: (ExprBinTimes with: 3 with: 2) with: 1.
        cell root: expr.
        sheet cellAtRow: 1 column: 1 put: cell.
        cell := CellExpression new.
        sheet cellAtRow: 2 column: 1 put: cell.
        sheet cellAtRow: 1 column: 2 put: 5.
        cell root: 
                (ExprFunSum new operands:
                        (CellReferenceRange new setReferenceFromR: 1 C: 1 toR: 1 C: 2 atSheet: sheet)
                 ).
        self assert: ((sheet cellAtRow:1 column:1) value = 7).
        self assert: (cell value = 12).
!

test05
        " tests the expressions' string representation "
        | sheet cell ref fun |

        " expressions with immediate values, test placement of parentheses
        around operands "
        cell := CellExpression new.
        cell root: (ExprBinPlus with: (ExprBinTimes with: 3 with: 2) with: 1).
        self assert: (cell stringRepr = '=3*2+1').

        cell root: (ExprBinTimes with: (ExprBinPlus with: 3 with: 2) with: (ExprUnNegation with: 1)).
        self assert: (cell stringRepr = '=(3+2)*(-1)').

        cell root: (ExprBinOver with: (ExprBinOver with: 1 with: 2) with: (ExprBinOver with: 3 with: 4)).
        self assert: (cell stringRepr = '=1/2/(3/4)').

        " expressions with references and functions "
        sheet := SpreadSheet new.
        ref := CellReference new setReferenceRow:1 column:3 atSheet:sheet.
        cell root: ref.
        self assert: (cell stringRepr = '=C1').

        cell root:
                (ExprFunSum new operands:
                        (CellReferenceRange new setReferenceFromR: 2 C: 1 toR: 5 C: 28 atSheet: sheet)
                 ).
        self assert: (cell stringRepr = '=sum(A2:AB5)').

        cell root:
                (ExprBinPlus with: 3 with:
                        (ExprFunSum new operands:
                                (CellReferenceRange new setReferenceFromR: 1 C:1 toR: 3 C:3 atSheet: sheet)
                        )
                 ).
        self assert: (cell stringRepr = '=3+sum(A1:C3)').

        cell root:
                (ExprBinPlus with:
                        (ExprFunSum new operands:
                                (CellReferenceRange new setReferenceFromR: 5 C:6 toR: 2 C:2 atSheet: sheet)
                        )
                 with: 5).
        self assert: (cell stringRepr = '=sum(B2:F5)+5').

        fun := ExprFunSum new operands: (OrderedCollection with: 1 with: 'foo' with: 5 with: 0.1).
        self assert: (fun stringRepr = 'sum(1,foo,5,0.1)').
! !

